from __future__ import unicode_literals

from django.db import models
from django.utils import timezone
from django.contrib.auth.models import User, AnonymousUser
import datetime


class Language (models.Model):
    name = models.CharField(max_length=20)
    image = models.ImageField()

    def __str__(self):
        return self.name

    def image_img(self):
        if self.image:
            return u'<img class="flag-icon" src="%s" />' % (self.image.url)
        else:
            return 'No image'

    image_img.short_description = 'Flag icon'
    image_img.allow_tags = True

class Subject(models.Model):
    SEX_CHOICES = (
        ('M','Male'),
        ('F','Female'),
    )
    sex = models.CharField(max_length=1,
        choices=SEX_CHOICES,
        verbose_name='Sex'
    )
    firstName = models.CharField(max_length=50,
        verbose_name='First name'
    )
    lastName = models.CharField(max_length=50,
        verbose_name='Last name'
    )
    languages = models.ManyToManyField(Language,
        verbose_name='Known languages'
    )
    phoneNo = models.CharField(max_length=20,
        verbose_name='Phone number'
    )
    email = models.EmailField(
        verbose_name='E-mail'
    )
    born = models.DateField(
        verbose_name='Date born'
    )
    address = models.CharField(max_length=255,
        verbose_name='Address'
    )
    postalCode = models.CharField(max_length=6,
        verbose_name='Postal code'
    )
    city = models.CharField(max_length=50,
        verbose_name='City'
    )
    country = models.CharField(max_length=50,
        verbose_name='Country'
    )
    screeningNo = models.CharField(max_length=50,
        verbose_name='Screening number'
    )
    ndno = models.CharField(max_length=6,
        verbose_name='ND number'
    )
    visitNumber = models.IntegerField(
        verbose_name='Number of visits'
    )
    mpowerId = models.CharField(max_length=20,
        verbose_name='MPower ID'
    )
    STATUS_CHOICES = (
        ('NCO', 'Not contacted'),
        ('TST', 'Test group'),
        ('LEA', 'Level A'),
        ('LEB', 'Level B'),
        ('OPT', 'Opted out'),
        ('DEA', 'Dead')
    )
    subjectStatus = models.CharField(max_length=3, choices=STATUS_CHOICES,
        verbose_name='Status',
        default='NCO'
    )
    mainPseudonym = models.CharField(max_length=45,
        verbose_name='Pseudonym'
    )
    comments = models.CharField(max_length=2000,
        verbose_name='Comments'
    )

    dateAdded = models.DateField(verbose_name='Added on',
        auto_now=True
    )

    referral = models.CharField(max_length=128,
        null=True,
        verbose_name='Referred by'
    )

    def __str__(self):
        return "%s %s" % (self.firstName, self.lastName)

    def __unicode__(self):
        return "%s %s" % (self.firstName, self.lastName)


class Visit(models.Model):
    subject = models.ForeignKey(Subject, on_delete=models.CASCADE,
        verbose_name='Subject'
    )
    visitBegin = models.DateTimeField(
        verbose_name='Visit starts at'
    )
    visitEnd = models.DateTimeField(
        verbose_name='Visit ends at'
    ) # Deadline before which all appointments need to be scheduled
    TYPE_CHOICES = (
        ('A', 'Level A visit'),
        ('B', 'Level B visit'),
        ('O', 'Other')
    )
    visitType = models.CharField(max_length=1, choices=TYPE_CHOICES,
        verbose_name='Visit type',
        default='O'
    )
    visitFinished = models.BooleanField(
        verbose_name='Has ended',
        default=False
    )

    def __str__(self):
        return "%s %s %d" % (self.subject.firstName, self.subject.lastName, self.assesmentNo)


class Item (models.Model):
    isFixed = models.BooleanField(
        default=False,
        verbose_name='Is the item fixed?'
    )
    name = models.CharField(max_length=255,
        verbose_name='Name'
    )

    def __str__(self):
        return self.name



class Room (models.Model):
    possEquipment = models.ManyToManyField(Item,
        verbose_name='On-site equipment',
        blank=True
    )
    owner = models.CharField(max_length=50,
        verbose_name='Owner'
    )
    address = models.CharField(max_length=255,
        verbose_name='Address'
    )
    city = models.CharField(max_length=50,
        verbose_name='City'
    )
    roomNo = models.IntegerField(
        verbose_name='Room number'
    )
    floor = models.IntegerField(
        verbose_name='Floor'
    )
    isVehicle = models.BooleanField(
        verbose_name='Is a vehicle?'
    )

    def __str__(self):
        return "%d %s %s" % (self.roomNo, self.address, self.city)

class AppointmentType (models.Model):
    requiredEquipment = models.ManyToManyField(Item,
        verbose_name='Required equipment'
    )
    code = models.CharField(max_length=20,
        verbose_name='Appointment code'
    )
    description = models.CharField(max_length=2000,
        verbose_name='Appointment description'
    )
    defaultTime = models.IntegerField(
        verbose_name='Default time'
    )
    restTime = models.IntegerField(
        verbose_name='Suggested rest time'
    )

    def __str__(self):
        return self.apCode

class Worker (models.Model):
    languages = models.ManyToManyField(Language,
        verbose_name='Known languages'
    )
    appointments = models.ManyToManyField('Appointment', blank=True,
        verbose_name='Appointments'
    )
    user = models.OneToOneField(User, blank=True, null=True,
        verbose_name='Username'
    )
    firstName = models.CharField(max_length=50,
        verbose_name='First name'
    )
    lastName = models.CharField(max_length=50,
        verbose_name='Last name'
    )
    phoneNo = models.CharField(max_length=20,
        verbose_name='Phone number'
    )
    unit = models.CharField(max_length=50,
        verbose_name='Unit'
    )
    email = models.EmailField(
        verbose_name='E-mail'
    )
    ROLE_CHOICES = (
        ('DOCTOR', 'Doctor'),
        ('NURSE', 'Nurse'),
        ('PSYCHOLOGIST', 'Psychologist'),
        ('TECHNICIAN', 'Technician'),
        ('SECRETARY', 'Secretary')
    )
    role = models.CharField(max_length=20, choices=ROLE_CHOICES,
        verbose_name='Role'
    )
    specialization = models.CharField(max_length=20,
        verbose_name='Specialization'
    )

    def isOnLeave(self):
        if len(self.holiday_set.filter(holidayEnd__gt=datetime.datetime.now(),
                                       holidayStart__lt=datetime.datetime.now())):
            return True
        return False

    def __str__(self):
        return "%s %s" % (self.firstName, self.lastName)

    @staticmethod
    def get_details(the_user):
        if the_user.is_authenticated == False:
            return ('Guest', 'Test account')

        person = Worker.objects.filter(user=the_user)

        if len(person) == 0:
            return ('Guest', 'Test account')
        else:
            # For get_*_display, see:
            # https://docs.djangoproject.com/en/1.10/topics/db/models/#field-options
            return (str(person[0]), person[0].get_role_display())


class FlyingTeam(models.Model):
    doctor = models.ForeignKey(Worker, related_name='FlyingTeamDoctor',
        verbose_name='Doctor'
    )
    nurse = models.ForeignKey(Worker, related_name='FlyingTeamNurse',
        verbose_name='Nurse'
    )
    psychologist = models.ForeignKey(Worker, related_name='FlyingTeamPsychologist',
        verbose_name='Psychologist'
    )
    dateTimeCalled = models.DateTimeField(
        verbose_name='Created on'
    )
    dateTimeUntil = models.DateTimeField(
        verbose_name='Disbanded on'
    )

    def __str__(self):
        return "%s %s %s" % (self.doctor.lastName, self.nurse.lastName, self.psychologist.lastName)

class Avaibility(models.Model):
    person = models.ForeignKey(Worker, on_delete=models.CASCADE,
        verbose_name='Worker'
    )
    dayNo = models.IntegerField(
        verbose_name='Day of the week'
    )
    availFro = models.TimeField(
        verbose_name='Avaible since'
    )
    availTil = models.TimeField(
        verbose_name='Avaible until'
    )
    current = models.BooleanField(
        verbose_name='Is current?',
        default=True
    )

    def __str__(self):
        return "%d %s %s" % (self.dayNo, self.person.lastName, self.person.firstName)

class Holiday(models.Model):
    person = models.ForeignKey(Worker, on_delete=models.CASCADE,
        verbose_name='Worker'
    )
    holidayStart = models.DateTimeField(
        verbose_name='On leave since'
    )
    holidayEnd = models.DateTimeField(
        verbose_name='On leave until'
    )

    def __str__(self):
        return "%s %s" % (self.person.firstName, self.person.lastName)

class Appointment(models.Model):
    flyingTeam = models.ForeignKey(FlyingTeam,
        verbose_name='Flying team (if applicable)',
        null=True, blank=True
    )
    workerAssigned = models.ForeignKey(Worker,
        verbose_name='Worker conducting the assessment (if applicable)',
        null=True, blank=True
    )
    appointmentType = models.ForeignKey(AppointmentType,
        verbose_name='Appointment type',
        null=True, blank=True
    )
    room = models.ForeignKey(Room,
        verbose_name='Room ID'
    )
    visit = models.ForeignKey(Visit,
        verbose_name='Visit ID'
    )
    appDateTime = models.DateTimeField(
        verbose_name='Appointment on'
    )
    appLength = models.IntegerField(
        verbose_name='Appointment length'
    )#Potentially redundant; but can be used to manually adjust appointment's length
    appFinished = models.BooleanField(
        verbose_name='Has the appointment ended?',
        default=False
    )
